"""Ensure version bump."""
import pkg_resources
import subprocess


def _any_py_files_changed(file_names):
    return any(file_name.endswith(".py") for file_name in file_names)


def _version_changed(pyproject_diff):
    return "+version = " in pyproject_diff


def find_remote():
    """Find the primary remote for a git repository."""
    remotes = subprocess.check_output(
        ["git", "remote"], universal_newlines=True
    ).splitlines()
    if len(remotes) == 1:
        return remotes[0]
    elif "origin" in remotes:
        return "origin"
    raise KeyError("Cannot determine remote")


def find_default_branch():
    """Find the default branch for a git repository."""
    remote = find_remote()
    remote_info = subprocess.check_output(
        ["git", "remote", "show", remote], universal_newlines=True
    ).splitlines()
    return [x for x in remote_info if "HEAD branch" in x][0].split(":")[1].strip()


def check_version():
    """Verify the version is changed if any code files are changed."""
    default_branch = find_default_branch()
    changed_files = subprocess.check_output(
        ["git", "diff", default_branch, "--name-only"], universal_newlines=True
    ).splitlines()

    pyproject_diff = subprocess.check_output(
        ["git", "diff", default_branch, "pyproject.toml"], universal_newlines=True
    )

    check_file_changes = _any_py_files_changed
    for entry in pkg_resources.iter_entry_points("file_checkers"):
        if entry.name == "version_trigger":
            check_file_changes = entry.load()
            break

    if check_file_changes(changed_files) and not _version_changed(pyproject_diff):
        print("Code files changed with no corresponding version bump!")
        exit(1)



"""Build the documentation for a package."""
import argparse
import email.utils
import itertools
import os
import pathlib
import shutil
import subprocess
import time
import warnings

from ..utils import CONFIGS, execute_command_list, get_pyproject_config
from .sample_conf import CONF_PY, MARKDOWN, TYPE_HINTS


__commands_to_run = CONFIGS["build_docs_commands"]
BASE_DIR = CONFIGS["base_dir"]
PACKAGE_NAME = CONFIGS["package_name"].replace("-", "_").replace(".", "_")

DOCS_OUTPUT_DIRECTORY = "docs"
DOCS_WORKING_DIRECTORY = "_docs"


def _build_docs():
    pyproject = get_pyproject_config()
    tools = pyproject["tool"].get("jgt_tools") or {}
    if "doc_build_types" in tools:
        message = (
            "The use of doc_build_types is deprecated and may be removed in a "
            "future version of the library. Use build_docs_commands instead."
        )
        warnings.warn(message, FutureWarning)
        if tools["doc_build_types"] == [] and "build_docs_commands" not in tools:
            return
    execute_command_list([x.format(**globals()) for x in __commands_to_run])


def _build_conf():
    docs_dir = BASE_DIR / DOCS_WORKING_DIRECTORY
    docs_dir.mkdir(exist_ok=True)

    pyproject = get_pyproject_config()
    poetry = pyproject["tool"]["poetry"]

    dependencies = list(
        itertools.chain(poetry["dependencies"], poetry["dev-dependencies"])
    )

    conf = CONF_PY.format(
        name=f"{poetry['name']} - {poetry['description']}",
        authors=", ".join(email.utils.parseaddr(x)[0] for x in poetry["authors"]),
        year=time.strftime("%Y"),
        type_hints=TYPE_HINTS if "sphinx-autodoc-typehints" in dependencies else "",
        version=poetry["version"],
        markdown=MARKDOWN if "recommonmark" in dependencies else "",
    )

    (docs_dir / "conf.py").write_text(conf)


def build():
    """Build the docs."""
    # Setup environment variables
    try:
        commit_id = subprocess.check_output(
            ["git", "rev-parse", "HEAD"],
            cwd=BASE_DIR,
            universal_newlines=True,
            stderr=subprocess.DEVNULL,
        )
    except subprocess.CalledProcessError:
        commit_id = ""
    os.environ["GIT_COMMIT_ID"] = commit_id.rstrip("\n")

    try:
        origin_url = subprocess.check_output(
            ["git", "config", "--get", "remote.origin.url"],
            cwd=BASE_DIR,
            universal_newlines=True,
            stderr=subprocess.DEVNULL,
        )
    except subprocess.CalledProcessError:
        origin_url = ""
    os.environ["GIT_ORIGIN_URL"] = origin_url.rstrip("\n")

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "--dirty",
        action="store_true",
        help="Leave the output directory untouched before starting to build documents",
    )
    args = parser.parse_args()

    if not args.dirty:
        shutil.rmtree(str(BASE_DIR / DOCS_OUTPUT_DIRECTORY), ignore_errors=True)
        shutil.rmtree(str(BASE_DIR / DOCS_WORKING_DIRECTORY), ignore_errors=True)

    _build_conf()

    _build_docs()


def push():
    """Push docs to github-pages."""
    if (pathlib.Path(BASE_DIR) / DOCS_OUTPUT_DIRECTORY).exists():
        subprocess.check_call(["poetry", "run", "ghp-import", "-p", "docs/"])


def build_and_push():
    """Build docs then publish."""
    build()
    push()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['jgt_tools', 'jgt_tools.docs']

package_data = \
{'': ['*'], 'jgt_tools': ['data/*']}

install_requires = \
['tomlkit>=0.7.0,<0.8.0']

extras_require = \
{'build_docs': ['sphinx>=3.1.2,<4.0.0',
                'sphinx-rtd-theme>=0.5.0,<0.6.0',
                'ghp-import>=0.5.5,<0.6.0'],
 'env_setup': ['pre-commit>=1.15,<2.0'],
 'run_tests': ['pytest>=5.0,<6.0']}

entry_points = \
{'console_scripts': ['build-and-push-docs = '
                     'jgt_tools.docs.build_docs:build_and_push[build_docs]',
                     'build-docs = jgt_tools.docs.build_docs:build[build_docs]',
                     'check-version = jgt_tools.check_version:check_version',
                     'env-setup = jgt_tools.env_setup:main',
                     'run-tests = jgt_tools.run_tests:main',
                     'self-check = jgt_tools.self_check:main']}

setup_kwargs = {
    'name': 'jgt-tools',
    'version': '0.4.2',
    'description': 'A collection of tools for commmon package scripts',
    'long_description': 'JGT Tools\n=========\n\nJGT Tools is a collection of package helpers\nfor common CLI functions\nwithin a properly-formatted repository.\n\n\nQuickstart\n----------\n\nJust include ``jgt_tools`` in your package VirtualEnv,\nand you\'ll have access to these CLI calls:\n\n- ``env-setup`` - set up the development environment\n  with all packages and pre-commit checks\n- ``self-check`` - run self-checks/linters/etc. on your repository\n- ``run-tests`` - run your in-repo test suite\n- ``build-docs`` - build repo documentation locally\n- ``build-and-push-docs`` - both build the docs,\n  then publish to your gh-pages branch\n- ``check-version`` - raise an error if package-relevant files have changed\n  without a version bump\n\nDetails for each script can be found by calling with the ``--help`` flag.\n\n.. tip::\n   In order to keep the environment as clean as possible,\n   JGT Tools will only install the libraries needed to start the tool itself.\n   However, several of the default commands\n   rely on additional libraries.\n   When ``env-setup`` is run for the first time,\n   it will check to see if each set of commands is still default,\n   and if so, install the libraries those commands need.\n\n\nrun-tests\n---------\nThe ``run-tests`` commands will\npass through any additional parameters\nprovided on the command line.\nFor example,\nby default ``run-tests`` maps to::\n\n    poetry run python -m pytest -vvv\n\nRunning ``run-tests -s`` would run::\n\n    poetry run python -m pytest -vvv -s\n\nDocumentation Index\n-------------------\n\nIn order to get the full benefit from ``build-docs``,\nit is encouraged to create an index file\nthat pulls together all the documentation.\nThis file needs to be in the root folder\nand should be called ``.jgt_tools.index``.\nThis will be moved into the working directory for Sphinx\nand be used when building the documentation.\nAdditional information can be found on the `Sphinx site`_.\n\nConfiguration\n-------------\n\nA number of the actions to be called\ncan be customized in a ``[tool.jgt_tools]``\nin your ``pyproject.toml`` file.\nAvailable values are:\n\n- ``env_setup_commands`` - a list of commands to be run\n  under the ``env-setup`` call\n- ``self_check_commands`` - a list of commands to be run\n  under the ``self-check`` call\n- ``run_tests_commands`` - a list of commands to be run\n  under the ``run-tests`` call\n- ``build_docs_commands`` - a list of commands to be run\n  under the ``build-docs`` call\n\nFor example::\n\n    [tool.jgt_tools]\n    env_setup_commands = [\n        "poetry install",\n        "poetry run pip install other_package",\n        "./my_custom_setup_script.sh"\n    ]\n    build_docs_commands = []\n\nwould run your specified commands for ``env-setup``\nand skip the default api doc builder.\n\n.. note::\n    NOTE: All commands provided in ``[tools.jgt_tools]``\n    will be run from project root.\n    To ensure your commands run as expected,\n    provide any paths in your custom commands relative from root.\n\nbuild_docs_commands\n~~~~~~~~~~~~~~~~~~~\n\nSpecifically for ``build_docs_commands``,\nthere are some variables\nthat can be used to aid in documentation building,\nusing Python-style curly-brace formatting::\n\n    BASE_DIR: Root library directory\n    PACKAGE_NAME: Folder name containing package\n    DOCS_WORKING_DIRECTORY: Temporary directory where docs are built (needed for Sphinx)\n    DOCS_OUTPUT_DIRECTORY: Final directory where docs are saved\n\nFor example::\n\n    [tool.jgt_tools]\n    build_docs_commands = [\n        "poetry run sphinx-apidoc --output-dir {DOCS_WORKING_DIRECTORY} --no-toc --force --module-first {PACKAGE_NAME}\n    ]\n\nbuilds the Sphinx API docs for the current package\nand stores the output files\nin the temporary working directory.\n\ncheck-version\n~~~~~~~~~~~~~\n\nIn addition,\nthe function to verify which files are relevant to ``check-version``\ncan be customized.\nBy default, if any files in the diff against master are ``.py`` files,\na version bump is expected,\nbut the user can provide an alternate function to verify filenames.\n\nThe function should expect a list of strings\nrepresenting file paths relative from project root\n(as provided by ``git diff master --name-only``)\nand return a boolean representing if a version change should be ensured\n(i.e. ``True`` if version should be checked).\n\nThis can be registered as a plugin in your ``pyproject.toml`` file::\n\n    [tools.poetry.plugins."file_checkers"]\n    "version_trigger" = "my_module:my_function"\n\nor in your ``setup.py`` file::\n\n    setup(\n        ...\n        entry_points={\n            "version_trigger": ["version_trigger = my_module:my_fuction"]\n        }\n    )\n\n.. _`Sphinx site`: http://www.sphinx-doc.org/en/master/usage/quickstart.html#defining-document-structure\n',
    'author': 'Brad Brown',
    'author_email': 'brad@bradsbrown.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://jolly-good-toolbelt.github.io/jgt_tools/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



"""
Run all available tests.

Runs the following commands:
    {}
"""
import argparse

from .utils import execute_command_list, CONFIGS
from .env_setup import env_setup
from .self_check import self_check as run_self_check

__commands_to_run = CONFIGS["run_tests_commands"]


__doc__ = __doc__.format("\n    ".join(__commands_to_run))


def run_tests(do_setup=False, self_check=False, verbose=True, additional_args=None):
    """Run code checks."""
    if do_setup:
        env_setup(verbose)
    if self_check:
        run_self_check(verbose=verbose)

    suffix = " {}".format(" ".join(additional_args)) if additional_args else ""
    execute_command_list([x + suffix for x in __commands_to_run], verbose=verbose)


def main():
    """Self check with cli args."""
    parser = argparse.ArgumentParser(
        formatter_class=argparse.RawDescriptionHelpFormatter, description=__doc__
    )
    parser.add_argument(
        "--setup",
        action="store_true",
        help='run "env-setup" before running self checks',
    )
    parser.add_argument(
        "--check", action="store_true", help='run "self-check" before running tests'
    )
    parser.add_argument(
        "-q",
        "--quiet",
        action="store_true",
        help="do not show each command before it is executed",
    )
    args, additional_args = parser.parse_known_args()

    run_tests(
        do_setup=args.setup,
        self_check=args.check,
        verbose=not args.quiet,
        additional_args=additional_args,
    )
